package org.netkernel.http.transport;

import java.util.Enumeration;

import org.eclipse.jetty.server.Connector;
import org.eclipse.jetty.server.ConnectorStatistics;
import org.eclipse.jetty.server.ServerConnector;
import org.eclipse.jetty.util.log.Logger;
import org.eclipse.jetty.xml.XmlConfiguration;
import org.netkernel.container.ILogger;
import org.netkernel.container.config.IConfiguration;
import org.netkernel.layer0.meta.IEndpointStateMeta;
import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardTransportImpl;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.impl.RequestResponseFieldsImpl;


public class HTTPTransportEndpoint extends StandardTransportImpl implements IEndpointStateMeta
{
	private NKContextServer mServer;
	private static String sServerString;
	
	public HTTPTransportEndpoint()
	{	declareSupportedVerbs(0);
		System.setProperty("JETTY_NO_SHUTDOWN_HOOK", Boolean.TRUE.toString());
	}
			
	@Override
	protected void postCommission(INKFRequestContext context) throws Exception
	{	context.logFormatted(INKFLocale.LEVEL_INFO,"MSG_COMMISSION_TRANSPORT");
		IConfiguration kcfg=context.getKernelContext().getKernel().getConfiguration();
		String product=kcfg.getString("netkernel.instance.product", "NetKernel");
		String major = kcfg.getString("netkernel.instance.version.major", "?");
		//String minor =kcfg.getString("netkernel.instance.version.minor", "?");
		sServerString=context.formatMessage("SERVER_STRING", product+" "+major);
		try
		{	IReadableBinaryStreamRepresentation serverConfig=(IReadableBinaryStreamRepresentation)context.source("param:config",IReadableBinaryStreamRepresentation.class);
			//final ILogger nkLogger = context.getKernelContext().getKernel().getLogger();
			final INKFRequestContext ctx=getTransportContext();
			Logger l=new Logger()
			{
				//This is a basic logger to satisfy Jetty but all logging is done either with an NCSA Handler (see XML config for example)
				//Or directly with the NK logging in the appropriate context - ie in a Handler or the HTTPBridge.
				public void debug(String arg0, Throwable arg1)
				{	ctx.logRaw(ILogger.DEBUG, arg0);
					ctx.logRaw(ILogger.DEBUG, Utils.throwableToString(arg1));
				}

				public Logger getLogger(String arg0)
				{	return this;
				}

				public boolean isDebugEnabled()
				{	return ctx.shouldLog(ILogger.DEBUG);
				}

				public void setDebugEnabled(boolean arg0)
				{	
				}

				public void warn(String arg0, Throwable arg1)
				{	ctx.logRaw(ILogger.WARNING, arg0);
					ctx.logRaw(ILogger.WARNING, Utils.throwableToString(arg1));
				}

				public void debug(Throwable arg0)
				{	this.debug(null, arg0);					
					
				}

				public void debug(String arg0, Object... arg1)
				{	ctx.logRaw(ILogger.DEBUG, arg0);
				}

				public String getName()
				{	return "HTTPTransport Logger";
				}

				public void ignore(Throwable arg0)
				{
				}

				public void info(Throwable arg0)
				{	//this.info(null, arg0);
				}

				public void info(String arg0, Object... arg1)
				{	//nkLogger.logRaw(ILogger.INFO, HTTPTransportEndpoint.this, arg0);
				}

				public void info(String arg0, Throwable arg1)
				{	//nkLogger.logRaw(ILogger.INFO, HTTPTransportEndpoint.this, arg0);
					//nkLogger.logRaw(ILogger.INFO, HTTPTransportEndpoint.this, Utils.throwableToString(arg1));
				}

				public void warn(Throwable arg0)
				{	this.warn(null, arg0);
				}

				public void warn(String arg0, Object... arg1)
				{	ctx.logRaw(ILogger.WARNING, arg0);
				}

				@Override
				public void debug(String arg0, long arg1)
				{	ctx.logRaw(ILogger.DEBUG, arg0 +" "+arg1);
					
				}
			};
			org.eclipse.jetty.util.log.Log.setLog(l);
		    mServer = new NKContextServer(getTransportContext());
		    
		    XmlConfiguration configuration = new XmlConfiguration(serverConfig.getInputStream()); 
		    configuration.configure(mServer);
		    
		    for(Connector i: mServer.getConnectors())
		    {   context.logFormatted(INKFLocale.LEVEL_INFO,"MSG_START_TRANSPORT", new Object[] {i.getName(), i.getProtocols() });
		    }
		    mServer.setStopAtShutdown(false);
		    mServer.start();
		    
		    //Initialize stats. - Now done with a bean added to Connector
		    /*
		    for(Connector c: mServer.getConnectors())
		    {	if(c.getStatsOn())
		    	{	c.statsReset();
		    	}
		    }
		    */

		}
		catch(Exception e)
		{	context.logFormatted(INKFLocale.LEVEL_WARNING,"EX_START_FAIL");
			e.printStackTrace();
			throw e;
		}
	}

	@Override
	protected void preDecommission(INKFRequestContext context) throws Exception
	{	context.logFormatted(INKFLocale.LEVEL_INFO,"MSG_DECOMMISSION_TRANSPORT");
		if(mServer!=null)
		{	//mServer.setGracefulShutdown(100);
			mServer.stop();
			Thread.sleep(200);
			mServer.destroy();
			for(INetKernelHandler handler : mServer.getNetKernelHandlers())
			{	handler.destroy();
			}
		}
		mContext=null;
	}
	
	static String getServerHeader()
	{	return sServerString;
	}

	public IRequestResponseFields getState()
	{	RequestResponseFieldsImpl fields=new RequestResponseFieldsImpl(super.getState());
		fields.put("serverstring", sServerString);
		int hid=0;
		if(mServer!=null)
		{
			for(INetKernelHandler handler : mServer.getNetKernelHandlers())
			{	String handlerid="handler"+hid+":";
				fields.put(handlerid+"name", handler.getClass().getName());
				fields.put(handlerid+"requests", handler.requestCount());
				hid++;
			}
			int cid=0;
			for(Connector c: mServer.getConnectors())
			{	String conid="con"+cid+":";
				fields.put(conid+"connector", c.getClass().getName());
				if(c instanceof ServerConnector)
				{
					ServerConnector sc=(ServerConnector)c;
					
					fields.put(conid+"port", sc.getPort());
					ConnectorStatistics stats=sc.getBean(ConnectorStatistics.class);
					if(stats!=null)
					{
						fields.put(conid+"Requests", stats.getMessagesIn());
						fields.put(conid+"Connections", stats.getConnections());
						fields.put(conid+"ConnectionsOpen", stats.getConnectionsOpen());
						fields.put(conid+"ConnectionsOpenMax", stats.getConnectionsOpenMax());
						//fields.put(conid+"ConnectionsRequestsAve", c.getConnectionsRequestsAve());
						fields.put(conid+"ConnectionsRequestsMax", stats.getMessagesInPerConnectionMax());
						fields.put(conid+"ConnectionsDurationAve", stats.getConnectionDurationMean());
						fields.put(conid+"ConnectionsDurationMax", stats.getConnectionDurationMax());
					}
					else
					{	fields.put(conid+"StatsNotSetOn", 0);
					}
				}
				cid++;
			}
		}
		return fields;
	}
}
