package org.netkernel.http.transport;

import java.io.IOException;
import java.net.URI;
import java.util.concurrent.atomic.AtomicInteger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.websocket.server.WebSocketHandler;
import org.eclipse.jetty.websocket.servlet.ServletUpgradeRequest;
import org.eclipse.jetty.websocket.servlet.ServletUpgradeResponse;
import org.eclipse.jetty.websocket.servlet.WebSocketCreator;
import org.eclipse.jetty.websocket.servlet.WebSocketServletFactory;
import org.netkernel.http.transport.representation.HTTPRequestResponseRepresentation;
import org.netkernel.http.transport.representation.NKWebSocket;
import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.util.Utils;

public class NetKernelWebSocketHandler extends WebSocketHandler implements INetKernelHandler 
{
	private INKFRequestContext mContext;
	private AtomicInteger mRequestCount=new AtomicInteger();
	private WebSocketServletFactory mFactory;
	
	public NetKernelWebSocketHandler()
	{	
	}
	
	@Override
	public void setServer(Server aServer)
	{	super.setServer(aServer);
		NKContextServer server=(NKContextServer)getServer();
		mContext=server.getTransportContext();
		server.registerNetKernelHandler(this);
		mContext.logFormatted(INKFLocale.LEVEL_INFO,"MSG_REGISTERED_WEBSOCKET_HANDLER", new Object[] {null});
	}

	@Override
	public void destroy()
	{	super.destroy();
		mContext=null;
	}
	
	@Override
	public int requestCount()
	{	return mRequestCount.get();
	}
	
	/*
	public WebSocket doWebSocketConnect(HttpServletRequest request, String protocol)
	{	
		NKWebSocket ws=null;
		try
		{	mRequestCount.incrementAndGet();
			String url=buildURL(request);
			ws=new NKWebSocket(url, request.getRemoteAddr(), protocol);
			HTTPRequestResponseRepresentation rep = new HTTPRequestResponseRepresentation(request,null, ws);
			INKFRequest req = mContext.createRequest(url);
			req.addPrimaryArgument(rep);
			mContext.issueRequest(req);		//Don't do it async otherwise WebSocket might not be bound to the WSTransport at the HTTPBridge level.
		}
		catch (Throwable t)
		{	t.printStackTrace();
		}
		return ws;
	}
	*/
	
	/*
	public boolean checkOrigin(HttpServletRequest request, String host)
	{	//You should override this method in your own handler if you want to perform host validation before constructing the socket.
		//The default is to initially allow the connection and then to implement host validation on the NEW request
		//to the ws: service provider using the primary argument HTTPReq/Rep representation 
		return true;
	}
	*/
	
	@Override
	public void handle(String target, Request baseRequest,
			HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException
	{
		if(mFactory.isUpgradeRequest(request, response))
		{	
			String url=buildURL(request);
			TransientNKWebSocketCreator c=new TransientNKWebSocketCreator(url, request.getRemoteAddr(), "ws");
			if(mFactory.acceptWebSocket(c, request, response))
			{
				System.out.println("WEBSOCKET UPGRADE ACCEPTED!!!!!!!!!!!!!!!!!!!!!!");
				//Now issue request to ROC domain
				try
				{	mRequestCount.incrementAndGet();
					HTTPRequestResponseRepresentation rep = new HTTPRequestResponseRepresentation(request,null, c.getNKWebSocket());
					INKFRequest req = mContext.createRequest(url);
					req.addPrimaryArgument(rep);
					mContext.issueRequest(req);		//Don't do it async otherwise WebSocket might not be bound to the WSTransport at the HTTPBridge level.
				}
				catch (Throwable t)
				{	t.printStackTrace();
				}
				baseRequest.setHandled(true);
				return;
			}
			// If we reach this point, it means we had an incoming request to upgrade
			// but it was either not a proper websocket upgrade, or it was possibly rejected
			// due to incoming request constraints (controlled by WebSocketCreator)
			if (response.isCommitted())
			{	// not much we can do at this point.
				System.out.println("FAILED TO WEBSOCKET UPGRADE!!!!!!!!!!!!!!!!!!!!!!");
				return;
			}
		}
	}

	private String buildURL(HttpServletRequest request)
	{	String url=Utils.decode(request.getRequestURL().toString());
		String qs=request.getQueryString();
		if (qs!=null)
		{	url=url+"?"+qs;
		}
		//url=url.replaceFirst("http", "ws");
		return url;
	}

	@Override
	public void configure(WebSocketServletFactory aFactory)
	{	mFactory=aFactory;
	}
	
	private class TransientNKWebSocketCreator implements WebSocketCreator
	{
		private String mURL;
		private String mRemoteAddress;
		private String mProtocol;
		
		private NKWebSocket mWebSocket;
		
		public TransientNKWebSocketCreator(String aURL, String aRemoteAddress, String aProtocol)
		{	mURL=aURL;
			mRemoteAddress=aRemoteAddress;
			mProtocol=aProtocol;
		}
		
		@Override
		public Object createWebSocket(ServletUpgradeRequest arg0,
				ServletUpgradeResponse arg1)
		{
			mWebSocket = new NKWebSocket(mURL, mRemoteAddress, mProtocol);
			return mWebSocket;
		}

		public NKWebSocket getNKWebSocket()
		{	return mWebSocket;
		}
	}
	
}
